@@ -44,9 +44,9 @@
 #include "yahoo-transport.h"
 #include "md5.h"
 #include "gaim-sha.h"
-#include "yahootransformtables.h"
+#include "yahoo-auth.h"
 
-extern char *yahoo_crypt(char *, char *);
+extern char *yahoo_crypt(const char *, const char *);
 
 //  #include "pixmaps/status-away.xpm"
 //  #include "pixmaps/status-here.xpm"
@@ -213,7 +213,7 @@ struct yahoo_packet *yahoo_packet_new(enum yahoo_service service, enum yahoo_sta
     return pkt;
 }
 
-static void yahoo_packet_hash(struct yahoo_packet *pkt, int key, char *value) {
+static void yahoo_packet_hash(struct yahoo_packet *pkt, int key, const char *value) {
     struct yahoo_pair *pair = g_new0(struct yahoo_pair, 1);
     pair->key = key;
     pair->value = g_strdup(value);
@@ -563,12 +563,17 @@ static void yahoo_process_message(struct yahoo_data *yd, struct yahoo_packet *pk
     char *msg = NULL;
     char *from = NULL;
     time_t tm = time(NULL);
+    int isUTF8 = 0;
     GSList *l = pkt->hash;
 
     while (l) {
         struct yahoo_pair *pair = l->data;
         if (pair->key == 4)
             from = pair->value;
+        if (pair->key == 97) {
+            if (strncasecmp(pair->value, "1", 1) == 0)
+               isUTF8 = 1;
+        }
         if (pair->key == 14)
             msg = pair->value;
         if (pair->key == 15)
@@ -592,7 +597,7 @@ static void yahoo_process_message(struct yahoo_data *yd, struct yahoo_packet *pk
             msg[j++] = m[i];
         }
         msg[j] = 0;
-        yahoo_send_jabber_message(yd, "chat", from, msg);
+        yahoo_send_jabber_message(yd, "chat", from, msg, isUTF8);
     } else if (pkt->status == 2) {
         // FUNC		do_error_dialog(_("Your message did not get sent."), _("Gaim - Error"));
     }
@@ -707,425 +712,457 @@ void to_y64(unsigned char *out, const unsigned char *in, int inlen) {
     *out = '\0';
 }
 
-
+#define SHA_CTX GAIM_SHA_CTX
+#define shaInit(a) gaim_shaInit(a)
+#define shaUpdate(a,b,c) gaim_shaUpdate((a),(b),(c))
+#define shaFinal(a,b) gaim_shaFinal((a),(b))
 static void yahoo_process_auth_new(struct yahoo_data *yd, const char *seed) {
     struct yahoo_packet *pack = NULL;
     const char *name = normalize(yd->username);
     const char *pass = yd->password;
 
-    md5_byte_t         result[16];
-    md5_state_t        ctx;
+	md5_byte_t			result[16];
+	md5_state_t			ctx;
 
-    GAIM_SHA_CTX            ctx1;
-    GAIM_SHA_CTX            ctx2;
+	SHA_CTX				ctx1;
+	SHA_CTX				ctx2;
 
-    char *alphabet1 = "FBZDWAGHrJTLMNOPpRSKUVEXYChImkwQ";
-    char *alphabet2 = "F0E1D2C3B4A59687abcdefghijklmnop";
+	char				*alphabet1			= "FBZDWAGHrJTLMNOPpRSKUVEXYChImkwQ";
+	char				*alphabet2			= "F0E1D2C3B4A59687abcdefghijklmnop";
 
-    char *challenge_lookup = "qzec2tb3um1olpar8whx4dfgijknsvy5";
-    char *operand_lookup = "+|&%/*^-";
-    char *delimit_lookup = ",;";
+	char				*challenge_lookup	= "qzec2tb3um1olpar8whx4dfgijknsvy5";
+	char				*operand_lookup		= "+|&%/*^-";
+	char				*delimit_lookup		= ",;";
 
-    char *password_hash = g_malloc0(25);
-    char *crypt_hash = g_malloc0(25);
-    char *crypt_result = NULL;
-    char pass_hash_xor1[64];
-    char pass_hash_xor2[64];
-    char crypt_hash_xor1[64];
-    char crypt_hash_xor2[64];
-    char resp_6[100];
-    char resp_96[100];
+	char				*password_hash		= (char *)g_malloc(25);
+	char				*crypt_hash		= (char *)g_malloc(25);
+	char				*crypt_result		= NULL;
 
-    unsigned char digest1[20];
-    unsigned char digest2[20];
-    unsigned char magic_key_char[4];
-    unsigned char *magic_ptr;
+	char				pass_hash_xor1[64];
+	char				pass_hash_xor2[64];
+	char				crypt_hash_xor1[64];
+	char				crypt_hash_xor2[64];
+	char				resp_6[100];
+	char				resp_96[100];
 
-    unsigned int  magic[64];
-    unsigned int  magic_work;
+	unsigned char		digest1[20];
+	unsigned char		digest2[20];
+	unsigned char		comparison_src[20]; 
+	unsigned char		magic_key_char[4];
+	const unsigned char		*magic_ptr;
 
-	char comparison_src[20]; 
-    int x;
-    int i,j;
-    int cnt = 0;
-    int magic_cnt = 0;
-    int magic_len;
-    int times = 0;
-    int magic_iterations = -1;
-    int magic_method = -1;
-    int magic_from_tables;
-        
-    unsigned char md5test[7];
+	unsigned int		magic[64];
+	unsigned int		magic_work = 0;
+	unsigned int		magic_4 = 0;
 
-    memset(&pass_hash_xor1, 0, 64);
-    memset(&pass_hash_xor2, 0, 64);
-    memset(&crypt_hash_xor1, 0, 64);
-    memset(&crypt_hash_xor2, 0, 64);
-    memset(&digest1, 0, 20);
-    memset(&digest2, 0, 20);
-    memset(&magic, 0, 64);
-    memset(&resp_6, 0, 100);
-    memset(&resp_96, 0, 100);
-    memset(&magic_key_char, 0, 4);
+	int					x;
+	int					y;
+	int					cnt = 0;
+	int					magic_cnt = 0;
+	int					magic_len;
 
-    /*
-     * Magic: Phase 1.  Generate what seems to be a 30
-     * byte value (could change if base64
-     * ends up differently?  I don't remember and I'm
-     * tired, so use a 64 byte buffer.
-     */
+	memset(password_hash, 0, 25);
+	memset(crypt_hash, 0, 25);
+	memset(&pass_hash_xor1, 0, 64);
+	memset(&pass_hash_xor2, 0, 64);
+	memset(&crypt_hash_xor1, 0, 64);
+	memset(&crypt_hash_xor2, 0, 64);
+	memset(&digest1, 0, 20);
+	memset(&digest2, 0, 20);
+	memset(&magic, 0, 64);
+	memset(&resp_6, 0, 100);
+	memset(&resp_96, 0, 100);
+	memset(&magic_key_char, 0, 4);
+	memset(&comparison_src, 0, 20);
 
-    magic_ptr = seed;
+	/* 
+	 * Magic: Phase 1.  Generate what seems to be a 30 byte value (could change if base64
+	 * ends up differently?  I don't remember and I'm tired, so use a 64 byte buffer.
+	 */
 
-    while (*magic_ptr != (int)NULL) {
-        char *loc;
-
-        /* Ignore parentheses.  */
-
-        if (*magic_ptr == '(' || *magic_ptr == ')') {
-            magic_ptr++;
-            continue;
-        }
-
-        /* Characters and digits verify against
-           the challenge lookup.
-        */
-
-        if (isalpha(*magic_ptr) || isdigit(*magic_ptr)) {
-            loc = strchr(challenge_lookup, *magic_ptr);
-            if (!loc) {
-                /* This isn't good */
-            }
-
-            /* Get offset into lookup table and lsh 3. */
-
-            magic_work = loc - challenge_lookup;
-            magic_work <<= 3;
-
-            magic_ptr++;
-            continue;
-        } else {
-            unsigned int local_store;
-
-            loc = strchr(operand_lookup, *magic_ptr);
-            if (!loc) {
-                /* Also not good. */
-            }
-
-            local_store = loc - operand_lookup;
-
-            /* Oops; how did this happen? */
-            if (magic_cnt >= 64)
-                break;
-
-            magic[magic_cnt++] = magic_work | local_store;
-            magic_ptr++;
-            continue;
-        }
-    }
-
-    magic_len = magic_cnt;
-    magic_cnt = 0;
-
-    /* Magic: Phase 2.  Take generated magic value and
-     * sprinkle fairy dust on the values. */
-
-    for (magic_cnt = magic_len-2; magic_cnt >= 0; magic_cnt--) {
-        unsigned char byte1;
-        unsigned char byte2;
-
-        /* Bad.  Abort.
-         */
-        if ((magic_cnt + 1 > magic_len) ||
-                    (magic_cnt > magic_len))
-            break;
-
-        byte1 = magic[magic_cnt];
-        byte2 = magic[magic_cnt+1];
-
-        byte1 *= 0xcd;
-        byte1 ^= byte2;
-
-        magic[magic_cnt+1] = byte1;
-    }
-
-	/* Magic: Phase 3.  This computes 20 bytes.  The first 4 bytes are used as our magic 
-	 * key (and may be changed later); the next 16 bytes are an MD5 sum of the magic key 
-	 * plus 3 bytes.  The 3 bytes are found by looping, and they represent the offsets 
-	 * into particular functions we'll later call to potentially alter the magic key. 
-	 * 
-	 * %-) 
-	 */ 
-
-    magic_cnt = 1;
-	x = 0; 
-
-	do { 
-		unsigned int     bl = 0;  
-		unsigned int     cl = magic[magic_cnt++]; 
-
-		if (magic_cnt >= magic_len) 
-            break;
-
-		if (cl > 0x7F) { 
-			if (cl < 0xe0)  
-				bl = cl = (cl & 0x1f) << 6;  
-        else {
-				bl = magic[magic_cnt++];  
-                              cl = (cl & 0x0f) << 6;  
-                              bl = ((bl & 0x3f) + cl) << 6;  
-            }
-
-			cl = magic[magic_cnt++];  
-			bl = (cl & 0x3f) + bl;  
-		} else 
-			bl = cl;  
-
-		comparison_src[x++] = (bl & 0xff00) >> 8;  
-		comparison_src[x++] = bl & 0xff;  
-	} while (x < 20); 
-	
-        /* First four bytes are the initial magic key. */
-	for (x = 0; x < 4; x++) 
-		magic_key_char[x] = comparison_src[x];
-
-        /* We need to determine the final magic key. When in mode 0, it's what we just
-         * calculated otherwise we'll need to use the big ugly lookup tables to calculate
-         * it. Many thanks to Jeffrey Lim for figuring this bit out.
-         */
-
-	memcpy(md5test, magic_key_char, 4);
-
-	for(i = 0; i < 0xFFFF; i++)
-	{
-		md5test[4] = i;
-		md5test[5] = i >> 8;
-
-		for(j = 0; j <= 5; j++)
-		{
-			md5test[6] = j;
-
-                        md5_init(&ctx);
-                        md5_append(&ctx, md5test, 7);
-                        md5_finish(&ctx, result);
-
-			if(!memcmp(&result, comparison_src+4, 16))
-			{
-				magic_iterations = i;
-				magic_method = j;
-        }
+	magic_ptr = seed;
 
+	while (*magic_ptr != (int)NULL) {
+		char   *loc;
+		
+		/* Ignore parentheses.
+		 */
+		
+		if (*magic_ptr == '(' || *magic_ptr == ')') {
+			magic_ptr++;
+			continue;
 		}
-    }
+		
+		/* Characters and digits verify against the challenge lookup.
+		 */
+		
+		if (isalpha(*magic_ptr) || isdigit(*magic_ptr)) {
+			loc = strchr(challenge_lookup, *magic_ptr);
+			if (!loc) {
+			  /* SME XXX Error - disconnect here */
+			}
+			
+			/* Get offset into lookup table and shl 3.
+			 */
+			
+			magic_work = loc - challenge_lookup;
+			magic_work <<= 3;
+			
+			magic_ptr++;
+			continue;
+		} else {
+			unsigned int	local_store;
+			
+			loc = strchr(operand_lookup, *magic_ptr);
+			if (!loc) {
+				/* SME XXX Disconnect */
+			}
+			
+			local_store = loc - operand_lookup;
+			
+			/* Oops; how did this happen?
+			 */
+			
+			if (magic_cnt >= 64) 
+				break;
+			
+			magic[magic_cnt++] = magic_work | local_store;
+			magic_ptr++;
+			continue;
+		}
+			}
+	
+	magic_len = magic_cnt;
+	magic_cnt = 0;
+	
+	/* Magic: Phase 2.  Take generated magic value and sprinkle fairy dust on the values.
+	 */
 
-   if (magic_method == 1) {
-		  unsigned int Salt = magic_key_char[0] | (magic_key_char[1]<<8) | (magic_key_char[2]<<16) | (magic_key_char[3]<<24);
-        magic_from_tables = yahoo_Dispatch(yahoo_Dispatch(Salt, magic_iterations), magic_iterations);
+	for (magic_cnt = magic_len-2; magic_cnt >= 0; magic_cnt--) {
+		unsigned char	byte1;
+		unsigned char	byte2;
+		
+		/* Bad.  Abort.
+		 */
+		
+		if ((magic_cnt + 1 > magic_len) || (magic_cnt > magic_len))
+			break;
+		
+		byte1 = magic[magic_cnt];
+		byte2 = magic[magic_cnt+1];
+		
+		byte1 *= 0xcd;
+		byte1 ^= byte2;
+		
+		magic[magic_cnt+1] = byte1;
+			}
+	
+	/* 
+	 * Magic: Phase 3.  This computes 20 bytes.  The first 4 bytes are used as our magic
+	 * key (and may be changed later); the next 16 bytes are an MD5 sum of the magic key
+	 * plus 3 bytes.  The 3 bytes are found by looping, and they represent the offsets
+	 * into particular functions we'll later call to potentially alter the magic key.
+	 *
+	 * %-)
+	 */
+	
+	magic_cnt = 1;
+	x = 0;
+	
+	do {
+		unsigned int	bl = 0; 
+		unsigned int	cl = magic[magic_cnt++];
+		
+		if (magic_cnt >= magic_len)
+			break;
+		
+		if (cl > 0x7F) {
+			if (cl < 0xe0) 
+				bl = cl = (cl & 0x1f) << 6; 
+			else {
+				bl = magic[magic_cnt++]; 
+				cl = (cl & 0x0f) << 6; 
+				bl = ((bl & 0x3f) + cl) << 6; 
+			} 
 
-        magic_key_char[0] = magic_from_tables & 0xff;
-        magic_key_char[1] = (magic_from_tables >> 8) & 0xff;
-        magic_key_char[2] = (magic_from_tables >> 16) & 0xff;
-        magic_key_char[3] = (magic_from_tables >> 24) & 0xff;
-    }
+			cl = magic[magic_cnt++]; 
+			bl = (cl & 0x3f) + bl; 
+		} else
+			bl = cl; 
+		
+		comparison_src[x++] = (bl & 0xff00) >> 8; 
+		comparison_src[x++] = bl & 0xff; 
+	} while (x < 20);
+	
+	/* First four bytes are magic key.
+	 */
+	
+	memcpy(&magic_key_char[0], comparison_src, 4);
+	magic_4 = magic_key_char[0] | (magic_key_char[1]<<8) | (magic_key_char[2]<<16) | (magic_key_char[3]<<24);
+	
+	/* 
+	 * Magic: Phase 4.  Determine what function to use later by getting outside/inside
+	 * loop values until we match our previous buffer.
+	 */
+	
+	for (x = 0; x < 65535; x++) {
+		int			leave = 0;
 
+		for (y = 0; y < 5; y++) {
+			md5_byte_t		result[16];
+			md5_state_t		ctx;
+			
+			unsigned char	test[3];
+			
+			memset(&result, 0, 16);
+			memset(&test, 0, 3);
+			
+			/* Calculate buffer.
+			 */
 
-    /* Get password and crypt hashes as per usual. */
-    md5_init(&ctx);
-    md5_append(&ctx, pass,  strlen(pass));
-    md5_finish(&ctx, result);
-    to_y64(password_hash, result, 16);
+			test[0] = x;
+			test[1] = x >> 8;
+			test[2] = y;
+			
+			md5_init(&ctx);
+			md5_append(&ctx, magic_key_char, 4);
+			md5_append(&ctx, test, 3);
+			md5_finish(&ctx, result);
+			
+			if (!memcmp(result, comparison_src+4, 16)) {
+				leave = 1;
+				break;
+			}
+		}
+		
+		if (leave == 1)
+			break;
+	}
+	
+	/* If y != 0, we need some help.
+	 */
+	
+	if (y != 0) {
+		unsigned int	updated_key;
+		
+		/* Update magic stuff.   Call it twice because Yahoo's encryption is super bad ass.
+		 */
+		
+		updated_key = yahoo_auth_finalCountdown(magic_4, 0x60, y, x);
+		updated_key = yahoo_auth_finalCountdown(updated_key, 0x60, y, x);
+		
+		magic_key_char[0] = updated_key & 0xff;
+		magic_key_char[1] = (updated_key >> 8) & 0xff;
+		magic_key_char[2] = (updated_key >> 16) & 0xff;
+		magic_key_char[3] = (updated_key >> 24) & 0xff;
+	} 
+	
+/* Get password and crypt hashes as per usual.
+	 */
+	
+	md5_init(&ctx);
+	md5_append(&ctx, pass, strlen(pass));
+	md5_finish(&ctx, result);
+	to_y64(password_hash, result, 16);
+	
+	md5_init(&ctx);
+	crypt_result = yahoo_crypt(pass, "$1$_2S43d5f$");  
+	md5_append(&ctx, crypt_result, strlen(crypt_result));
+	md5_finish(&ctx, result);
+	to_y64(crypt_hash, result, 16);
 
-    md5_init(&ctx);
-    crypt_result = yahoo_crypt(pass, "$1$_2S43d5f$");
-    md5_append(&ctx, crypt_result, strlen(crypt_result));
-    md5_finish(&ctx, result);
-    to_y64(crypt_hash, result, 16);
+	/* Our first authentication response is based off of the password hash.
+	 */
+	
+	for (x = 0; x < (int)strlen(password_hash); x++) 
+		pass_hash_xor1[cnt++] = password_hash[x] ^ 0x36;
+	
+	if (cnt < 64) 
+		memset(&(pass_hash_xor1[cnt]), 0x36, 64-cnt);
 
-    /* Our first authentication response is based off
-     * of the password hash. */
+	cnt = 0;
+	
+	for (x = 0; x < (int)strlen(password_hash); x++) 
+		pass_hash_xor2[cnt++] = password_hash[x] ^ 0x5c;
+	
+	if (cnt < 64) 
+		memset(&(pass_hash_xor2[cnt]), 0x5c, 64-cnt);
+	
+	shaInit(&ctx1);
+	shaInit(&ctx2);
+	
+	/* 
+	 * The first context gets the password hash XORed with 0x36 plus a magic value
+	 * which we previously extrapolated from our challenge.
+	 */
+	
+	shaUpdate(&ctx1, pass_hash_xor1, 64);
+	if (y >= 3)
+		ctx1.sizeLo = 0x1ff;
+	shaUpdate(&ctx1, magic_key_char, 4);
+	shaFinal(&ctx1, digest1);
+	
+	/* 
+	 * The second context gets the password hash XORed with 0x5c plus the SHA-1 digest
+	 * of the first context.
+	 */
+	
+	shaUpdate(&ctx2, pass_hash_xor2, 64);
+	shaUpdate(&ctx2, digest1, 20);
+	shaFinal(&ctx2, digest2);
+	
+	/* 
+	 * Now that we have digest2, use it to fetch characters from an alphabet to construct
+	 * our first authentication response.
+	 */
 
-    for (x = 0; x < (int)strlen(password_hash); x++)
-        pass_hash_xor1[cnt++] = password_hash[x] ^ 0x36;
+	for (x = 0; x < 20; x += 2) {
+		unsigned int	val = 0;
+		unsigned int	lookup = 0;
+		
+		char			byte[6];
+				
+		memset(&byte, 0, 6);
 
-    if (cnt < 64)
-        memset(&(pass_hash_xor1[cnt]), 0x36, 64-cnt);
+		/* First two bytes of digest stuffed together.
+		 */
 
-    cnt = 0;
+		val = digest2[x];
+		val <<= 8;
+		val += digest2[x+1];
+		
+		lookup = (val >> 0x0b);
+		lookup &= 0x1f;
+		if (lookup >= strlen(alphabet1))
+			break;
+		sprintf(byte, "%c", alphabet1[lookup]);
+		strcat(resp_6, byte);
+		strcat(resp_6, "=");
 
-    for (x = 0; x < (int)strlen(password_hash); x++)
-        pass_hash_xor2[cnt++] = password_hash[x] ^ 0x5c;
+		lookup = (val >> 0x06);
+		lookup &= 0x1f;
+		if (lookup >= strlen(alphabet2))
+			break;
+		sprintf(byte, "%c", alphabet2[lookup]);
+		strcat(resp_6, byte);
+		
+		lookup = (val >> 0x01);
+		lookup &= 0x1f;
+		if (lookup >= strlen(alphabet2))
+			break;
+		sprintf(byte, "%c", alphabet2[lookup]);
+		strcat(resp_6, byte);
 
-    if (cnt < 64)
-        memset(&(pass_hash_xor2[cnt]), 0x5c, 64-cnt);
+		lookup = (val & 0x01);
+		if (lookup >= strlen(delimit_lookup))
+			break;
+		sprintf(byte, "%c", delimit_lookup[lookup]);
+		strcat(resp_6, byte);
+	}
+	
+	/* Our second authentication response is based off of the crypto hash.
+	 */
+	
+	cnt = 0;
+	memset(&digest1, 0, 20);
+	memset(&digest2, 0, 20);
+	
+	for (x = 0; x < (int)strlen(crypt_hash); x++) 
+		crypt_hash_xor1[cnt++] = crypt_hash[x] ^ 0x36;
+	
+	if (cnt < 64) 
+		memset(&(crypt_hash_xor1[cnt]), 0x36, 64-cnt);
 
-    gaim_shaInit(&ctx1);
-    gaim_shaInit(&ctx2);
+	cnt = 0;
+	
+	for (x = 0; x < (int)strlen(crypt_hash); x++) 
+		crypt_hash_xor2[cnt++] = crypt_hash[x] ^ 0x5c;
+	
+	if (cnt < 64) 
+		memset(&(crypt_hash_xor2[cnt]), 0x5c, 64-cnt);
+	
+	shaInit(&ctx1);
+	shaInit(&ctx2);
+	
+	/* 
+	 * The first context gets the password hash XORed with 0x36 plus a magic value
+	 * which we previously extrapolated from our challenge.
+	 */
+	
+	shaUpdate(&ctx1, crypt_hash_xor1, 64);
+	if (y >= 3)
+		ctx1.sizeLo = 0x1ff;
+	shaUpdate(&ctx1, magic_key_char, 4);
+	shaFinal(&ctx1, digest1);
+	
+	/* 
+	 * The second context gets the password hash XORed with 0x5c plus the SHA-1 digest
+	 * of the first context.
+	 */
+	
+	shaUpdate(&ctx2, crypt_hash_xor2, 64);
+	shaUpdate(&ctx2, digest1, 20);
+	shaFinal(&ctx2, digest2);
+	
+	/* 
+	 * Now that we have digest2, use it to fetch characters from an alphabet to construct
+	 * our first authentication response.
+	 */
+	
+	for (x = 0; x < 20; x += 2) {
+		unsigned int	val = 0;
+		unsigned int	lookup = 0;
+		
+		char			byte[6];
+		
+		memset(&byte, 0, 6);
+		
+		/* First two bytes of digest stuffed together.
+		 */
+		
+		val = digest2[x];
+		val <<= 8;
+		val += digest2[x+1];
 
-    /* The first context gets the password hash XORed
-     * with 0x36 plus a magic value
-     * which we previously extrapolated from our
-     * challenge. */
+		lookup = (val >> 0x0b);
+		lookup &= 0x1f;
+		if (lookup >= strlen(alphabet1))
+			break;
+		sprintf(byte, "%c", alphabet1[lookup]);
+		strcat(resp_96, byte);
+		strcat(resp_96, "=");
+		
+		lookup = (val >> 0x06);
+		lookup &= 0x1f;
+		if (lookup >= strlen(alphabet2))
+			break;
+		sprintf(byte, "%c", alphabet2[lookup]);
+		strcat(resp_96, byte);
+		
+		lookup = (val >> 0x01);
+		lookup &= 0x1f;
+		if (lookup >= strlen(alphabet2))
+			break;
+		sprintf(byte, "%c", alphabet2[lookup]);
+		strcat(resp_96, byte);
+		
+		lookup = (val & 0x01);
+		if (lookup >= strlen(delimit_lookup))
+			break;
+		sprintf(byte, "%c", delimit_lookup[lookup]);
+		strcat(resp_96, byte);
+	}
+	
+	pack = yahoo_packet_new(YAHOO_SERVICE_AUTHRESP,	YAHOO_STATUS_AVAILABLE, 0);
+	yahoo_packet_hash(pack, 0, name);
+	yahoo_packet_hash(pack, 6, resp_6);
+	yahoo_packet_hash(pack, 96, resp_96);
+	yahoo_packet_hash(pack, 1, name);
+	yahoo_send_packet(yd, pack);
+	yahoo_packet_free(pack);
 
-    gaim_shaUpdate(&ctx1, pass_hash_xor1, 64);
-    gaim_shaUpdate(&ctx1, magic_key_char, 4);
-    gaim_shaFinal(&ctx1, digest1);
-
-    /* The second context gets the password hash XORed
-     * with 0x5c plus the SHA-1 digest
-     * of the first context. */
-
-    gaim_shaUpdate(&ctx2, pass_hash_xor2, 64);
-    gaim_shaUpdate(&ctx2, digest1, 20);
-    gaim_shaFinal(&ctx2, digest2);
-
-    /* Now that we have digest2, use it to fetch
-     * characters from an alphabet to construct
-     * our first authentication response. */
-
-    for (x = 0; x < 20; x += 2) {
-        unsigned int    val = 0;
-        unsigned int    lookup = 0;
-        char            byte[6];
-
-        memset(&byte, 0, 6);
-
-        /* First two bytes of digest stuffed
-         *  together.
-         */
-
-        val = digest2[x];
-        val <<= 8;
-        val += digest2[x+1];
-
-        lookup = (val >> 0x0b);
-        lookup &= 0x1f;
-        if (lookup >= strlen(alphabet1))
-            break;
-        sprintf(byte, "%c", alphabet1[lookup]);
-        strcat(resp_6, byte);
-        strcat(resp_6, "=");
-
-        lookup = (val >> 0x06);
-        lookup &= 0x1f;
-        if (lookup >= strlen(alphabet2))
-            break;
-        sprintf(byte, "%c", alphabet2[lookup]);
-        strcat(resp_6, byte);
-
-        lookup = (val >> 0x01);
-        lookup &= 0x1f;
-        if (lookup >= strlen(alphabet2))
-            break;
-        sprintf(byte, "%c", alphabet2[lookup]);
-        strcat(resp_6, byte);
-
-        lookup = (val & 0x01);
-        if (lookup >= strlen(delimit_lookup))
-            break;
-        sprintf(byte, "%c", delimit_lookup[lookup]);
-        strcat(resp_6, byte);
-    }
-
-    /* Our second authentication response is based off
-     * of the crypto hash. */
-
-    cnt = 0;
-    memset(&digest1, 0, 20);
-    memset(&digest2, 0, 20);
-
-    for (x = 0; x < (int)strlen(crypt_hash); x++)
-        crypt_hash_xor1[cnt++] = crypt_hash[x] ^ 0x36;
-
-    if (cnt < 64)
-        memset(&(crypt_hash_xor1[cnt]), 0x36, 64-cnt);
-
-    cnt = 0;
-
-    for (x = 0; x < (int)strlen(crypt_hash); x++)
-        crypt_hash_xor2[cnt++] = crypt_hash[x] ^ 0x5c;
-
-    if (cnt < 64)
-        memset(&(crypt_hash_xor2[cnt]), 0x5c, 64-cnt);
-
-    gaim_shaInit(&ctx1);
-    gaim_shaInit(&ctx2);
-
-    /* The first context gets the password hash XORed
-     * with 0x36 plus a magic value
-     * which we previously extrapolated from our
-     * challenge. */
-
-    gaim_shaUpdate(&ctx1, crypt_hash_xor1, 64);
-    gaim_shaUpdate(&ctx1, magic_key_char, 4);
-    gaim_shaFinal(&ctx1, digest1);
-
-    /* The second context gets the password hash XORed
-     * with 0x5c plus the SHA-1 digest
-     * of the first context. */
-
-    gaim_shaUpdate(&ctx2, crypt_hash_xor2, 64);
-    gaim_shaUpdate(&ctx2, digest1, 20);
-    gaim_shaFinal(&ctx2, digest2);
-
-    /* Now that we have digest2, use it to fetch
-     * characters from an alphabet to construct
-     * our first authentication response.  */
-
-    for (x = 0; x < 20; x += 2) {
-        unsigned int val = 0;
-        unsigned int lookup = 0;
-
-        char byte[6];
-
-        memset(&byte, 0, 6);
-
-        /* First two bytes of digest stuffed
-         *  together. */
-
-        val = digest2[x];
-        val <<= 8;
-        val += digest2[x+1];
-
-        lookup = (val >> 0x0b);
-        lookup &= 0x1f;
-        if (lookup >= strlen(alphabet1))
-            break;
-        sprintf(byte, "%c", alphabet1[lookup]);
-        strcat(resp_96, byte);
-        strcat(resp_96, "=");
-
-        lookup = (val >> 0x06);
-        lookup &= 0x1f;
-        if (lookup >= strlen(alphabet2))
-            break;
-        sprintf(byte, "%c", alphabet2[lookup]);
-        strcat(resp_96, byte);
-
-        lookup = (val >> 0x01);
-        lookup &= 0x1f;
-        if (lookup >= strlen(alphabet2))
-            break;
-        sprintf(byte, "%c", alphabet2[lookup]);
-        strcat(resp_96, byte);
-
-        lookup = (val & 0x01);
-        if (lookup >= strlen(delimit_lookup))
-            break;
-        sprintf(byte, "%c", delimit_lookup[lookup]);
-        strcat(resp_96, byte);
-    }
-
-    pack = yahoo_packet_new(YAHOO_SERVICE_AUTHRESP, YAHOO_STATUS_AVAILABLE, 0);
-    yahoo_packet_hash(pack, 0, name);
-    yahoo_packet_hash(pack, 6, resp_6);
-    yahoo_packet_hash(pack, 96, resp_96);
-    yahoo_packet_hash(pack, 1, name);
-
-    yahoo_send_packet(yd, pack);
-
-    free(password_hash);
-    free(crypt_hash);
-    yahoo_packet_free(pack);
+	g_free(password_hash);
+	g_free(crypt_hash);
 }
 
 
@@ -1645,12 +1682,12 @@ void yahoo_act_id(struct yahoo_data *yd, char *entry) {
 //  }
 
 int yahoo_send_im(struct yahoo_data *yd, char *who, char *what, int len, int flags) {
-    struct yahoo_packet *pkt = yahoo_packet_new(YAHOO_SERVICE_MESSAGE, YAHOO_STATUS_OFFLINE, 0);
+    struct yahoo_packet *pkt = yahoo_packet_new(YAHOO_SERVICE_MESSAGE, YAHOO_STATUS_AVAILABLE, 0);
 
     yahoo_packet_hash(pkt, 1, yd->displayname);
     yahoo_packet_hash(pkt, 5, who);
     yahoo_packet_hash(pkt, 14, what);
-
+    yahoo_packet_hash(pkt, 97, "1");
     yahoo_send_packet(yd, pkt);
 
     yahoo_packet_free(pkt);
@@ -1700,7 +1737,7 @@ void yahoo_set_away(struct yahoo_data *yd, int state, char *msg) {
         yahoo_packet_hash(pkt, 19, msg);
     }
 
-    log_notice(ZONE, "[YAHOO]: Presence service=0x%04x status=0x%04x", service,yd->current_status);
+//  log_notice(ZONE, "[YAHOO]: Presence service=0x%04x status=0x%04x", service,yd->current_status);
     yahoo_send_packet(yd, pkt);
     yahoo_packet_free(pkt);
 }
